% Load the digit sample data

datasetPath = '.\digits_images'; % specify your own path
digitData = imageDatastore(datasetPath, ...
        'IncludeSubfolders',true,'LabelSource','foldernames');

% Check the number of images in each category. 
countLabel = digitData.countEachLabel;

figure
%fprintf("Displaying sample images & labels\n")
perm = randperm(10000,20);
for i = 1:20
    subplot(4,5,i);
    imshow(digitData.Files{perm(i)});
    text(16, 32, digitData.Labels(perm(i)));
end

trainNum = 750;  % for each label
rng(1) % For reproducibility
[trainDigitData,testDigitData] = splitEachLabel(digitData, ...
				trainNum,'randomize'); 

fprintf("Training files count: %d\n", length(trainDigitData.Labels))
fprintf("Testing files count:  %d\n", length(testDigitData.Labels))

% Define the convolutional neural network architecture. 
layers = [imageInputLayer([28 28 1])
          convolution2dLayer(5,20)
          reluLayer
          maxPooling2dLayer(2,'Stride',2)
          fullyConnectedLayer(10)
          softmaxLayer
          classificationLayer()];
      
% Specify the Training Options
options = trainingOptions('sgdm','MaxEpochs',15, ...
	'InitialLearnRate',0.0001);

% Train the Network Using Training Data
convnet = trainNetwork(trainDigitData,layers,options);

% Classify the Images in the Test Data and Compute Accuracy
yTest = classify(convnet,testDigitData);
tTest = testDigitData.Labels;

% Calculate the accuracy. 
accuracy = sum(yTest == tTest)/numel(tTest)

figure
%fprintf("Displaying true labels vs predictions\n")
perm = randperm(2500,20);
for i = 1:20
    subplot(4,5,i);
    imshow(testDigitData.Files{perm(i)});
    text(1, 32, tTest(perm(i)));
    if tTest(perm(i)) ~= yTest(perm(i))
        text(16, 32, yTest(perm(i)), 'Color', 'r')
        text(20, 32, '***', 'Color', 'r')
    else
        text(16, 32, yTest(perm(i)));
    end
end